#include "convertSimMCContributionsToHitCollection.h"

#include <stdexcept>


#include "UTIL/BitSet32.h"
#include "EVENT/SimCalorimeterHit.h"
#include "IMPL/CalorimeterHitImpl.h"


convertSimHitsStepToCaloHit::convertSimHitsStepToCaloHit(std::vector<std::string> collectionNames, std::ostream& errorstream)
  : m_inputCollections(),
    m_inputCollectionsNames(collectionNames),
    m_error(errorstream),
    m_CellIDMask(0xffffffff),
    m_CellIDEncoding("")
{
  setILDmode();
}

void convertSimHitsStepToCaloHit::setILDmode()
{
  m_CellIDMask=0xffffffff;
  m_CellIDEncoding="system:0:5,module:5:3,stave:8:4,tower:12:5,layer:17:6,slice:23:4";
  m_CellIDEncoding+=",PDGshowerContribution:32:-32";
}


LCCollectionVec* convertSimHitsStepToCaloHit::convert(const EVENT::LCEvent* evt)
{
  LCCollectionVec* outputCollection=nullptr;
  prepareCollection(evt);
  convertSimMCContributionsToHitCollection(outputCollection);
  return outputCollection;
}

void convertSimHitsStepToCaloHit::prepareCollection(const EVENT::LCEvent* evt)
{
  m_inputCollections.clear();
  for (std::string& collectionName : m_inputCollectionsNames)
    {
      LCCollection* SimHitCollection=nullptr;
      try {SimHitCollection=evt->getCollection( collectionName.c_str() ); }
      catch(lcio::DataNotAvailableException &e )
	{
	  m_error << "convertSimHitsStepToCaloHit::prepareCollection() : collection named " << collectionName << " not available in the event" << std::endl;
	  SimHitCollection=nullptr;
	}
      if (SimHitCollection != nullptr)
	m_inputCollections.push_back(SimHitCollection);
    }
}


void convertSimHitsStepToCaloHit::convertSimMCContributionsToHitCollection(LCCollectionVec* &outputCollection)
{
  outputCollection=new LCCollectionVec( LCIO::CALORIMETERHIT );
  outputCollection->setFlag( UTIL::make_bitset32( LCIO::RCHBIT_LONG, /* position will be stored*/
						  LCIO::RCHBIT_ID1, /* cellid 1 will be stored and contain PDG code of step contribution*/
						  LCIO::RCHBIT_TIME, /* time info will be stored*/
						  LCIO::RCHBIT_ENERGY_ERROR /* energy error will be used to store step length*/ ));
  outputCollection->parameters().setValue( LCIO::CellIDEncoding,  m_CellIDEncoding );
  for ( LCCollection* inputCol : m_inputCollections)
    {
      if ( inputCol->getTypeName() != LCIO::SIMCALORIMETERHIT )
	throw std::logic_error("convertSimMCContributionsToHitCollection : input collection should contain SimCalorimeterHit");
      int flag=inputCol->getFlag();
      if ( not UTIL::BitSet32( flag )[LCIO::CHBIT_STEP] )
	throw std::logic_error("convertSimMCContributionsToHitCollection : collection should contain detailed shower information (i.e. step information)");
      for (int indexSimHit=0; indexSimHit<inputCol->getNumberOfElements(); ++indexSimHit)
	{
	  SimCalorimeterHit* simHit=dynamic_cast<SimCalorimeterHit*> ( inputCol->getElementAt(indexSimHit) );
	  if (simHit==0)
	    throw std::logic_error("convertSimMCContributionsToHitCollection : problem in SimCalorimeterHit conversion");
	  int cellID0=(simHit->getCellID0() & m_CellIDMask);
	  for (int indexMCstep=0; indexMCstep<simHit->getNMCContributions(); ++indexMCstep)
	    {
	      CalorimeterHitImpl* calHit = new CalorimeterHitImpl;
	      calHit->setCellID0(cellID0);
	      calHit->setCellID1( simHit->getPDGCont(indexMCstep) );
	      calHit->setEnergy( simHit->getEnergyCont(indexMCstep) );
	      calHit->setEnergyError( simHit->getLengthCont(indexMCstep) );
	      calHit->setTime( simHit->getTimeCont(indexMCstep) );
	      calHit->setPosition( simHit->getStepPosition(indexMCstep) );
	      calHit->setRawHit( simHit->getParticleCont(indexMCstep) );
	      outputCollection->addElement( calHit );
	    }
	}
    }

}
